#!/usr/bin/perl
##
# Fetcher for Amazon's's pages.
#

package TopFilms::Amazon;

use TopFilms;
@ISA = ('TopFilms');



##
# Our name.
sub name
{
    return "Amazon";
}


##
# Amazon has a crazy way of handling multiple pages.
# Each page is a separate URL, as you might expect, but within each page the
# content is fetched as 2 requests - one for 'above the fold' and one for below.
# So to get the full charts, we have to fetch the lot.
sub expandurls
{
    my ($code) = @_;
    # Note the 'FOO' in the URL appears to be quite unimportant - the bit that
    # matters is the number.
    my $prefix = 'http://www.amazon.co.uk/Best-Sellers-DVD-Blu-ray-FOO/zgbs/dvd/';
    my $suffix = '/?_encoding=UTF8&ajax=1&';
    my @list;
    
    for my $pagenum (1..5)
    {
        for my $abovethefold (1, 0)
        {
            my $url = "$prefix$code$suffix";
            $url .= "pg=$pagenum&isAboveTheFold=$abovethefold";
            push @list, $url
        }
    }
    
    return \@list;
}

##
# Return the charts that we handle.
sub getCharts
{
    return {
            'Action' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Action and Adventure',
                    'url' => expandurls('501778'),
                },
            'Children' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => "Children's",
                    'url' => expandurls('501858'),
                },
            'Classics' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Classics',
                    'url' => expandurls('501976'),
                },
            'Comedy' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Comedy',
                    'url' => expandurls('501866'),
                },
            'Crime+Mystery' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Crime and Mystery',
                    'url' => expandurls('501880'),
                },
            'Drama' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Drama',
                    'url' => expandurls('501872'),
                },
            'Musicals' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Musicals',
                    'url' => expandurls('1108824'),
                },
            'SciFi' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Science Fiction',
                    'url' => expandurls('501916'),
                },
            'Fantasy' =>
                {
                    'charset' => 'iso-8859-1',
                    'name' => 'Fantasy',
                    'url' => expandurls('501914'),
                },
        };
}

##
# The titles used by Amazon are just untidy - they truncate wildly.
# We need to clean these names up so that they're at least slightly useful.
sub cleanup
{
    my ($entry) = @_;
    my $name = $entry->{'title'};

    $name =~ s/ \(.*?\)$//gi;
    $name =~ s/ \[region free\]//gi;
    $name =~ s/ \[DVD( *\+.*?)?\]//g;
    $name =~ s/ \(DVD( *\+.*?)?\)//g;
    $name =~ s/ \[blu-ray( *(3D *)?\+.*?)?\]//gi;
    $name =~ s/ \(blu-ray( *(3D *)?\+.*?)?\)//gi;
    $name =~ s/ blu-ray( with digital copy)?$//gi;
    $name =~ s/ - \d[ -] ?Disc edition//gi;
    $name =~ s/ \[\d[ -] ?Disc edition\]//gi;
    $name =~ s/ (\d-\d |\d DVD |\d disc )?box ?set//gi;
    $name =~ s/ Trilogy$//gi;
    $name =~ s/ Quadrilogy$//gi;
    $name =~ s/ - *$//gi;
    $name =~ s/ - triple (play|pack).*$//gi;
    $name =~ s/ - the original$//gi;
    $name =~ s/ - (Extreme|director's|special|collectors) edition//gi; #'
    $name =~ s/ - (the )?(complete )?(\d+|[a-z]+) Disc Set//gi;
    $name =~ s/ (\d+|[a-z]+) Disc//gi;
    $name =~ s/ special edition//gi;
    $name =~ s/ directors cut//gi;

    # Now remove the truncated parameters if any    
    $name =~ s/ \([^\)]*\.\.\.$//;
    $name =~ s/ \[[^\]]*\.\.\.$//;
    # Leave on the truncation marker so we can see it was truncated
    #$name =~ s/ ?\.\.\.$//;
    
    # Strip any date
    $name =~ s/ \[\d\d\d\d\]//;
    $name =~ s/ \(\d\d\d\d\)//;
    
    $entry->{'title'} = $name;
    
    return $entry;
}


##
# Patterns to apply for a chart's page.
sub pattern
{
    return {
            'entry'     => qr/(<div class="zg_rankDiv">.*?<div class="zg_reviews">)/s,
            'number'    => qr/<span class="zg_rankNumber">(\d+).<\/span>/s,
            'title'     => qr/<img src="[^"]+" alt="(.*?)"/s, # Name from "s
            'img'       => qr/<img src="([^"]+)" alt="(.*?)"/s,  # Name from "s
            'clean'     => \&cleanup,
        };
}


# Modules must return true.
1;
