#!/usr/bin/perl
##
# Fetcher for BFI's pages.
#
# BFI produced a 'top 100 films' list in 1999 from a survey of 1000 people
# from the business.
# They produced a 'top 50 films' list in 2012 which updated this.
#
# The original list from 1999 is no longer on their site, but is on the BBC
# site.
#

package TopFilms::BFI;

use TopFilms;
@ISA = ('TopFilms');



##
# Our name.
sub name
{
    return "BFI";
}


##
# Return the charts that we handle.
sub getCharts
{
    return {
            '1999' =>
                {
                    'name' => 'Top 100 British Films (1999)',
                    # Yup, we'll fetch from the BBC because it's not on the BFI
                    # site.
                    'url' => 'http://news.bbc.co.uk/1/hi/entertainment/455170.stm',
                },

            '2012' =>
                {
                    'name' => 'Top 50 (2012)',
                    'url' => 'http://www.bfi.org.uk/news/50-greatest-films-all-time',
                },
        };
}


##
# Patterns to apply for a chart's page.
sub pattern1999
{
    # Gosh this is ugly:
    #<UL>1 The Third Man (1949, Carol Reed)<BR> 
    #2 Brief Encounter (1945, David Lean)
    #<br>
    # 3 Lawrence of Arabia (1962, David Lean)
    #<br>
    # 4 The 39 Steps (1935, Alfred Hitchcock)<br>  5 Great Expectations (1946, David Lean)<br>  6 Kind Hearts and Coronets (1949, Robert Hamer)
    #<P>
    #  7 Kes (1969 Ken Loach)  <br>8 Don't Look Now (1973, Nicolas Roeg) <br> 9 The Red Shoes (1948, Michael Powell, Emeric Pressburger) <br> 10 Trainspotting (1996, Danny Boyle) 
    #<P>

    # I'm going to take a shot at just splitting on <br> and <p> and matching
    # the pattern of numbers.
    return {
            'split'     => qr/<(?:br|p)>/i,
            'number'    => qr/\b(\d+) [A-Z0-9][^\(<]+ \(\d\d\d\d,? [^\)<]+\)/,
            'title'     => qr/\b\d+ ([A-Z0-9][^\(<]+) \(\d\d\d\d,? [^\)<]+\)/,
            'year'      => qr/\b\d+ [A-Z0-9][^\(<]+ \((\d\d\d\d),? [^\)<]+\)/,
        };
}

##
# Patterns to apply for a chart's page.
sub pattern2012
{
    # The BFI 50 files for 2012 is WAY easier though:
    # <h3>17=<a href="http://explore.bfi.org.uk/4ce2b6b2deaa5">Persona</a></h3>
    # <p> 
    # <div class="image full">
    #   <img class="lazy lazy-hidden" data-aspect-ratio="1.32584269663" data-src="http://www.bfi.org.uk/sites/bfi.org.uk/files/styles/full/public/image/8-half-1963-001-00m-l57-guido-atop-car_590.jpg?itok=gSFkiKEp" alt="" title="" /><noscript><img  data-aspect-ratio="1.32584269663" src="http://www.bfi.org.uk/sites/bfi.org.uk/files/styles/full/public/image/8-half-1963-001-00m-l57-guido-atop-car_590.jpg?itok=gSFkiKEp" alt="" title="" /></noscript>  </div>
    # </p>
    # <p><em>Federico Fellini, 1963</em> (64 votes)</p>

    return {
            'entry'     => qr/<h3>(\d.*)<\/h3>/,
            'number'    => qr/^(\d+)[\.=] <a href=".*?">.*?<\/a>/,
            'title'     => qr/^\d+[\.=] <a href=".*?">(.*?)<\/a>/,
            'year'      => qr/, (\d\d\d\d)<\/em> \(\d+ vote/,
        };
}

# Modules must return true.
1;
