#!/usr/bin/perl
##
# Fetcher for Film4's pages.
#
#

package TopFilms::Film4;

use TopFilms;
@ISA = ('TopFilms');



##
# Our name.
sub name
{
    return "Film 4";
}


##
# Return the charts that we handle.
sub getCharts
{
    return {
            'BeforeYouDie' =>
                {
                    'name' => '50 films to see before you die',
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-50-films-to-see-before-you-die?component-id=400881137&module-id=400881136&module-type=FL01&ajax-page-timestamp=1402251254016&ajax-page='. $_ } (1..5) ],
                },

            'Comedies' =>
                {
                    'name' => 'Top 30 comedies', # Really, it has 30, not 25.
                    'url' => [ map { 'http://www.film4.com/special-features/top-100-films/top-25-comedies?component-id=300456520&module-id=300456519&module-type=FL01&ajax-page-timestamp=1402257911919&ajax-page='. $_ } (1..3) ],
                },

            'Romantic' =>
                {
                    'name' => 'Top 40 Romantic films', # The items 41-50 are copies of 31-40.
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-50-romantic-films?component-id=400881398&module-id=400881397&module-type=FL01&ajax-page-timestamp=1402257826273&ajax-page='. $_ } (1..4) ],
                },

            'SciFi' =>
                {
                    'name' => 'Top 50 Sci-Fi films',
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-50-sci-fi-movies?component-id=400880964&module-id=400880963&module-type=FL01&ajax-page-timestamp=1402257654490&ajax-page='. $_ } (1..5) ],
                },

            'Thrillers' =>
                {
                    'name' => 'Top 30 Thrillers',
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-30-thrillers?component-id=400881091&module-id=400881090&module-type=FL01&ajax-page-timestamp=1402255934837&ajax-page='. $_ } (1..3) ],
                },

            'Musicals' =>
                {
                    'name' => 'Top 50 Musicals',
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-50-musicals?component-id=400881316&module-id=400881314&module-type=FL01&ajax-page-timestamp=1402257064241&ajax-page='. $_ } (1..5) ],
                },

            'Animation' =>
                {
                    'name' => 'Top 20 Animated films', # Top 20, not top 30 (last 10 is a copy of the first 10)
                    'url' => [ map { 'http://www.film4.com/special-features/top-lists/top-25-animated-films?component-id=400881248&module-id=400881247&module-type=FL01&ajax-page-timestamp=1402257156613&ajax-page='. $_ } (1..2) ],
                },

        };
}


##
# Patterns to apply for a chart's page.
sub pattern
{
    # Initially this was just for the BeforeYouDie page, but they all appear to have
    # the same pattern.

    # Lines that have the data in that we care about look like this:
    # <h3><a href="/reviews/1987/withnail-and-i">Withnail & I (1987) Bruce Robinson</a></h3>
    # EXCEPT when it comes to this one:
    # <h3><a href="/reviews/1979/monty-pythons-life-of-brian">Monty Python's Life Of Brian, 1979: Terry Jones</a></h3>
    # AND some entries don't have links.
    # AND Donnie Darko is missing the space between the 'title' and '(year)'.
    # AND we have "Hairspray (1988/2007) John Waters/Adam Shankman"

    # We don't appear to have any numbers in the returned results, so we'll
    # rely on the fact that we generate them automatically.
    return {
            'entry'     => qr/<li (.*?)<\/li>/is,
            # No number
            'title'     => qr/<h3>(?:<a href=".*?">)?([^<]+?)(?: ?\(\d\d\d\d[\)\/]|, \d\d\d\d:)/,
            # The year match doesn't work for Life Of Brian because of the format difference -
            # the match is in the second captured expression, not the first.
            'year'      => qr/<h3>(?:<a href=".*?">)?[^<]+?(?: ?\((\d\d\d\d)[\)\/]|, (\d\d\d\d):)/,
            'url'       => qr/<h3><a href="(.*?)">[^<]+?(?: ?\(\d\d\d\d[\)\/]|, \d\d\d\d:)/,
        };
}


# Modules must return true.
1;
