#!/usr/bin/perl
##
# Fetcher for TotalFilm's pages.
#

package TopFilms::TotalFilm;

use TopFilms;
@ISA = ('TopFilms');



##
# Our name.
sub name
{
    return "Total Film";
}


##
# Return the charts that we handle.
sub getCharts
{
    return {
            'BTTM' => {
                    'name' => 'Best Time Travel Movies',
                    'url'  => 'http://www.totalfilm.com/features/50-best-time-time-travel-movies/',
                },
        };
}


##
# Patterns to apply for a chart's page.
#
# This is used rather than chartBTTM if it is present; we return just the
# patterns to process as regular expressions.
sub patternBTTM
{
    # Entries take the form (without breaks):
    # <li data-number="0">
    # <a href="/features/50-best-time-time-travel-movies/the-butterfly-effect-2004" id="imglink_0" class="thumbImage" title="The Butterfly Effect (2004)">
    # <div class="img_wrap">
    # <img src="http://cdn.mos.totalfilm.com/images/t/the-butterfly-effect-2004--02.jpg" id="img_1" alt="The Butterfly Effect (2004)" />
    # <span class="number">50</span>
    # </div>
    # </a>
    # </li>

    return {
            'entry'     => qr/(<li data-number.*?<\/li>)/,
            'number'    => qr/<span class="number">(\d+)<\/span>/,
            'title'     => qr/<img src="[^"]+" id="[^"]+" alt="([^"]+?)(?: \(\d\d\d\d\))?" \/>/, # Name from "s
            'year'      => qr/<img src="[^"]+" id="[^"]+" alt="[^"]+ \((\d\d\d\d)\)" \/>/,       # Name from "s
            'img'       => qr/<img src="([^"]+)" id="[^"]+" alt="[^"]+(?: \(\d\d\d\d\))?" \/>/,  # Name from "s
        };
}


##
# Fetcher for the chart.
# (NOTE: Not used; the above pattern routine will be used instead)
#
# @param[in] $page  The fetched page
#
# @return arrayref containing the list details, each entry being a hashref of
#           'number'    Number in the list (optional - if not given they are
#                       assumed to be incrementing from 1)
#           'title'     The film title
#           'year'      The film year (optional)
#           'url'       Any URL that might have been given (optional)
#           'img'       Any image URL that might have been given (optional)
sub chartBTTM
{
    my ($self, $page) = @_;
    my $result = [];
    
    # Entries take the form (without breaks):
    # <li data-number="0">
    # <a href="/features/50-best-time-time-travel-movies/the-butterfly-effect-2004" id="imglink_0" class="thumbImage" title="The Butterfly Effect (2004)">
    # <div class="img_wrap">
    # <img src="http://cdn.mos.totalfilm.com/images/t/the-butterfly-effect-2004--02.jpg" id="img_1" alt="The Butterfly Effect (2004)" />
    # <span class="number">50</span>
    # </div>
    # </a>
    # </li>
    for my $li ($page =~ /(<li data-number.*?<\/li>)/g)
    {
        my ($number, $title, $year, $url, $img);
        ($number) = ($li =~ /<span class="number">(\d+)<\/span>/);
        ($title) = ($li =~ /<img src="[^"]+" id="[^"]+" alt="([^"]+?)(?: \(\d\d\d\d\))?" \/>/);
        ($year) = ($li =~ /<img src="[^"]+" id="[^"]+" alt="[^"]+ \((\d\d\d\d)\)" \/>/);
        ($img) = ($li =~ /<img src="([^"]+)" id="[^"]+" alt="[^"]+(?: \(\d\d\d\d\))?" \/>/);
        # Extract from "s

        if (defined $number && defined $title)
        {
            push @$result, {
                    'number' => $number,
                    'title'  => $title,
                    defined $year ? ('year' => $year) : (),
                    defined $url ? ('url' => $url) : (),
                    defined $img ? ('img' => $img) : (),
                };
        }
    }
    
    return $result;
}


# DEBUG: Was used for testing
if (0)
{
    use Data::Dumper;
    use XML::Simple;
    $site = new TopFilms::TotalFilm();
    my $chart = $site->chart('BTTM');
    print XML::Simple::XMLout( { 'title' => 'TITLE HERE',
                                 'film' => $chart },
                               KeepRoot => 1,
                               RootName => 'chart' );
    #print Dumper($chart);
}


# Modules must return true.
1;